@@ -3,6 +3,7 @@ from __future__ import annotations
 from abc import ABCMeta, abstractmethod
 from selectors import BaseSelector
 from ssl import SSLSession
+from http import HTTPStatus
 from weakref import WeakValueDictionary
 from typing import List, Dict, Tuple, Union, Callable, Optional, Any, NoReturn, Iterable, Iterator
 # Third party modules
@@ -40,9 +41,12 @@ class T_Socket(metaclass=ABCMeta):
 	def getsockname(self) -> T_SocketBindAddress: ...
 
 class SSLContext(metaclass=ABCMeta):
+	@abstractmethod
 	def wrap_socket(self, sock:T_Socket, server_side:bool=..., do_handshake_on_connect:bool=..., suppress_ragged_eofs:bool=...,
 	server_hostname:Optional[str]=..., session:Optional[SSLSession]=...) -> T_Socket: ...
 
+class NoPayload: ...
+
 class T_Client(metaclass=ABCMeta):
 	max_bulk_request:int
 	FSPACKER_VERSION:int
@@ -83,17 +87,19 @@ class T_Client(metaclass=ABCMeta):
 	@abstractmethod
 	def _connect(self, sendOlderRequests:bool=...) -> None: ...
 	@abstractmethod
-	def _sendRequest(self, data:Any, path:str=..., extraHttpHeaders:Optional[Dict[str, str]]=...) -> None: ...
+	def _sendRequest(self, data:Any, httpMethod:str=..., path:str=..., httpHeaders:Dict[str, str]=...) -> None: ...
 	@abstractmethod
-	def _sendToSocket(self, payload:bytes, path:str=..., extraHttpHeaders:Dict[str, str]=...) -> None: ...
+	def _sendToSocket(self, payload:bytes, httpMethod:str=..., path:str=..., httpHeaders:Dict[str, str]=...) -> None: ...
 	@abstractmethod
 	def _createSocket(self) -> None: ...
 	@abstractmethod
 	def _get(self, id:Any) -> None: ...
 	@abstractmethod
-	def _parseResponse(self, payload:bytes, headers:Optional[T_Headers]=..., charset:str=...) -> None: ...
+	def _parseResponse(self, payload:bytes, headers:Optional[T_Headers]=..., charset:str=...,
+	httpStatus:Optional[HTTPStatus]=...) -> None: ...
 	@abstractmethod
-	def _parseResult(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str) -> None: ...
+	def _parseResult(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str,
+	httpStatus:Optional[HTTPStatus]=...) -> None: ...
 	@abstractmethod
 	def clear(self) -> None: ...
 	@abstractmethod
@@ -101,8 +107,8 @@ class T_Client(metaclass=ABCMeta):
 	@abstractmethod
 	def close(self) -> None: ...
 	@abstractmethod
-	def request(self, method:str, args:List[Any]=..., kwargs:Dict[str, Any]=..., id:Optional[Union[str, int]]=...,
-	path:str=...) -> T_Request: ...
+	def request(self, method:str=..., args:List[Any]=..., kwargs:Dict[str, Any]=..., id:Optional[Union[str, int]]=...,
+	path:Optional[str]=..., httpMethod:str=..., httpHeaders:Optional[Dict[str, str]]=..., payload:Any=...) -> T_Request: ...
 
 class T_BaseClientSocket(metaclass=ABCMeta):
 	client:T_Client
@@ -163,13 +169,14 @@ class T_BaseClientSocket_send_default(T_BaseClientSocket, metaclass=ABCMeta):
 
 class T_BaseClientSocket_send_http(T_BaseClientSocket, metaclass=ABCMeta):
 	@abstractmethod
-	def send(self, payload:bytes=b"", path:str=..., headers:Dict[str, str]=...) -> None: ...
+	def send(self, payload:bytes=b"", httpMethod:str=..., path:str=..., headers:Dict[str, str]=...) -> None: ...
 
 class T_HTTPClientSocket(T_BaseClientSocket_send_http, metaclass=ABCMeta):
 	defaultHeaders:Dict[str, str]
 	headers:T_Headers
+	path:str
 	@abstractmethod
-	def send(self, payload:bytes=b"", path:str=..., headers:Dict[str, str]=...) -> None: ...
+	def send(self, payload:bytes=b"", httpMethod:str=..., path:Optional[str]=..., headers:Dict[str, str]=...) -> None: ...
 
 class T_StringClientSocket(T_BaseClientSocket_send_default, metaclass=ABCMeta):
 	pass
@@ -217,9 +224,13 @@ class T_Request(metaclass=ABCMeta):
 	_args:List[Any]
 	_kwargs:Dict[Any, Any]
 	_path:str
+	_httpMethod:str
+	_httpHeaders:Dict[str, str]
+	_payload:str
 	_convertNumbers:Optional[str]
 	_requestTime:float
 	_responseTime:float
+	_httpStatus:Optional[HTTPStatus]
 	_uid:str
 	_done:bool
 	_success:bool
@@ -227,7 +238,8 @@ class T_Request(metaclass=ABCMeta):
 	@abstractmethod
 	def _get(self) -> None: ...
 	@abstractmethod
-	def _parseResponse(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str) -> None: ...
+	def _parseResponse(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str,
+	httpStatus:Optional[HTTPStatus]=...) -> None: ...
 	@abstractmethod
 	def _dumps(self) -> Any: ...
 	@abstractmethod
@@ -242,3 +254,5 @@ class T_Request(metaclass=ABCMeta):
 	def getUID(self) -> str: ...
 	@abstractmethod
 	def isSuccess(self) -> bool: ...
+	@abstractmethod
+	def getHTTPStatus(self) -> Optional[HTTPStatus]: ...
