@@ -1,6 +1,7 @@
 # Builtin modules
 from __future__ import annotations
 import json, weakref, pickle
+from http import HTTPStatus
 from lz4.frame import decompress # type: ignore
 from typing import Any, Union, List, Dict, Tuple, Optional, Callable, cast
 # Third party modules
@@ -8,22 +9,22 @@ import fsPacker
 from fsLogger import Logger, T_Logger
 from fsSignal import HardSignal, KillSignal, T_Signal
 # Local modules
-from .exceptions import InitializationError, MessageError, RequestError, ResponseError, BaseRPCError
+from .exceptions import InitializationError, RequestError, ResponseError, BaseRPCError
 from .utils import iterSplit
 from .clientSocket import HTTPClientSocket, StringClientSocket, FSPackerClientSocket, OldFSProtocolClientSocket
 from .abcs import (T_Request, T_Client, T_Headers, T_HTTPClientSocket, T_BaseClientSocket_send_default, T_SocketBindAddress,
-T_BaseClientSocket_send_http, T_StringClientSocket, T_FSPackerClientSocket, T_OldFSPackerClientSocket)
+T_BaseClientSocket_send_http, T_StringClientSocket, T_FSPackerClientSocket, T_OldFSPackerClientSocket, NoPayload)
 from .requestObj import Request
 # Program
 _dumpRequest:Callable[[T_Request], Any] = lambda x: x._dumps()
 
 class Client(T_Client):
-	max_bulk_request:int = 0xFF
+	max_bulk_request:int = 100
 	FSPACKER_VERSION:int = fsPacker.HIGHEST_VERSION
 	def __init__(self, protocol:str, target:Union[str, Tuple[str, int], Tuple[str, int, int, int]],
 	connectTimeout:Union[int, float]=15, transferTimeout:Union[int, float]=320, retryCount:int=10,
 	retryDelay:Union[int, float]=5, ssl:bool=False, sslHostname:Optional[str]=None, httpHost:Optional[str]=None,
-	extraHttpHeaders:Dict[str, str]={}, disableCompression:bool=False, useBulkRequest:bool=True,
+	extraHttpHeaders:Dict[str, str]={}, path:str="/", disableCompression:bool=False, useBulkRequest:bool=True,
 	convertNumbers:Optional[str]=None, bind:Optional[T_SocketBindAddress]=None, log:Optional[T_Logger]=None,
 	signal:Optional[T_Signal]=None) -> None:
 		self.protocol           = protocol
@@ -36,6 +37,7 @@ class Client(T_Client):
 		self.sslHostname        = sslHostname
 		self.httpHost           = httpHost
 		self.extraHttpHeaders   = extraHttpHeaders
+		self.path               = path
 		self.disableCompression = disableCompression
 		self.useBulkRequest     = useBulkRequest
 		self.convertNumbers     = convertNumbers
@@ -71,18 +73,20 @@ class Client(T_Client):
 			self.target = cast(Tuple[str, int], self.target)
 			if not isinstance(self.target, (tuple, list)) or len(self.target) != 2 or \
 			not isinstance(self.target[0], str) or not isinstance(self.target[1], int):
-				raise InitializationError("Target must be address and port in a list\tuple")
+				raise InitializationError("Target must be address and port in a list or tuple")
 		elif self.socketProtocol == "TCPv6":
 			self.target = cast(Tuple[str, int, int, int], self.target)
 			if  not isinstance(self.target, (tuple, list)) or len(self.target) != 4 or \
 			not isinstance(self.target[0], str) or not isinstance(self.target[1], int) or \
 			not isinstance(self.target[2], int) or not isinstance(self.target[3], int):
-				raise InitializationError("Target must be address, port, flow info and scope id in a list\tuple")
+				raise InitializationError("Target must be address, port, flow info and scope id in a list or tuple")
 		else:
 			raise InitializationError("Unsupported socket protocol")
 		if self.messageProtocol not in ["HTTP", "STR", "FSP", "OldFSProtocol"]:
 			raise InitializationError("Unsupported message protocol")
-		if self.requestProtocol not in ["JSONRPC-2", "JSONRPC-P", "FSP", "OldFSProtocol"]:
+		if self.requestProtocol in ("REST", "RAW"):
+			self.useBulkRequest = False
+		elif self.requestProtocol not in ("JSONRPC-2", "JSONRPC-P", "FSP", "OldFSProtocol"):
 			raise InitializationError("Unsupported request protocol")
 		if self.messageProtocol == "OldFSProtocol":
 			if self.requestProtocol != "OldFSProtocol":
@@ -111,6 +115,7 @@ class Client(T_Client):
 			"sslHostname":       self.sslHostname,
 			"httpHost":          self.httpHost,
 			"extraHttpHeaders":  self.extraHttpHeaders,
+			"path":              self.path,
 			"disableCompression":self.disableCompression,
 			"useBulkRequest":    self.useBulkRequest,
 			"convertNumbers":    self.convertNumbers,
@@ -129,6 +134,7 @@ class Client(T_Client):
 		self.sslHostname        = states["sslHostname"]
 		self.httpHost           = states["httpHost"]
 		self.extraHttpHeaders   = states["extraHttpHeaders"]
+		self.path               = states["path"]
 		self.disableCompression = states["disableCompression"]
 		self.useBulkRequest     = states["useBulkRequest"]
 		self.convertNumbers     = states["convertNumbers"]
@@ -141,36 +147,53 @@ class Client(T_Client):
 		self.socket.connect()
 		self.log.info("Connected")
 		if sendOlderRequests:
+			obj:T_Request
 			objs:List[T_Request] = list(filter(lambda x: not x.isDone(), self.requests.values()))
 			if objs:
 				objs.sort(key=lambda x: x._requestTime)
 				self.log.info("Sending {} previous requests", len(objs))
-				chunkToSend:Any
 				if self.useBulkRequest and len(objs) > 1:
-					for chunk in iterSplit(list(map(_dumpRequest, objs)), self.max_bulk_request):
-						self._sendRequest(chunk)
-				else:
+					# Ossze kell valogatni httpMethod es path alapjan
+					chunks:Dict[Tuple[str, str, str], List[Any]] = {}
 					for obj in objs:
-						self._sendRequest(obj._dumps(), path=obj._path)
-	def _sendRequest(self, data:Any, path:str="/", extraHttpHeaders:Optional[Dict[str, str]]=None) -> None:
-		payload:bytes
+						p:Tuple[str, str, str] = (obj._httpMethod, obj._path, json.dumps(obj._httpHeaders, sort_keys=True))
+						if p not in chunks:
+							chunks[p] = []
+						chunks[p].append(obj._dumps())
+					for (httpMethod, path, jsonHttpHeaders), allChunksData in chunks.items():
+						for chunkData in self.signal.iter(iterSplit(allChunksData, self.max_bulk_request)):
+							self._sendRequest(chunkData, httpMethod, path, json.loads(jsonHttpHeaders))
+				else:
+					for obj in self.signal.iter(objs):
+						self._sendRequest(obj._dumps(), obj._httpMethod, obj._path, obj._httpHeaders)
+	def _sendRequest(self, data:Any, httpMethod:str="POST", path:str="/", httpHeaders:Dict[str, str]={}) -> None:
+		payload:bytes = b""
 		if isinstance(self.socket, T_HTTPClientSocket):
-			extraHttpHeaders = extraHttpHeaders or {}
-			extraHttpHeaders["host"] = self.httpHost or "{}:{}".format(self.target[0], self.target[1])
-			if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P"]:
-				extraHttpHeaders["Content-Type"] = "application/json;charset=utf-8"
-				payload = json.dumps(data).encode('utf8')
-			elif self.requestProtocol == "FSP":
-				extraHttpHeaders["Content-Type"] = "application/fspacker"
-				payload = fsPacker.dumps(data, version=self.FSPACKER_VERSION)
-			else:
-				raise RuntimeError
-			self._sendToSocket(payload, path, extraHttpHeaders)
+			if "host" not in httpHeaders:
+				httpHeaders["host"] = self.httpHost or "{}:{}".format(self.target[0], self.target[1])
+			if data is not NoPayload:
+				if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P", "REST"]:
+					if "content-type" not in httpHeaders:
+						httpHeaders["content-type"] = "application/json;charset=utf-8"
+					payload = json.dumps(data).encode('utf8')
+				elif self.requestProtocol == "FSP":
+					if "content-type" not in httpHeaders:
+						httpHeaders["content-type"] = "application/fspacker"
+					payload = fsPacker.dumps(data, version=self.FSPACKER_VERSION)
+				elif self.requestProtocol == "RAW":
+					assert type(data) is bytes
+					payload = data
+				else:
+					raise RuntimeError
+			self._sendToSocket(payload, httpMethod, path, httpHeaders)
 		elif isinstance(self.socket, T_FSPackerClientSocket):
-			if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P"]:
+			if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P", "REST"]:
 				payload = json.dumps(data).encode('utf8')
 			elif self.requestProtocol == "FSP":
 				payload = fsPacker.dumps(data, version=self.FSPACKER_VERSION)
+			elif self.requestProtocol == "RAW":
+				assert type(data) is bytes
+				payload = data
 			else:
 				raise RuntimeError
 			self._sendToSocket(payload)
@@ -181,18 +204,23 @@ class Client(T_Client):
 				raise RuntimeError
 			self._sendToSocket(payload)
 		elif isinstance(self.socket, T_StringClientSocket):
-			if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P"]:
+			if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P", "REST"]:
 				payload = json.dumps(data).encode('utf8')
 			elif self.requestProtocol == "FSP":
 				payload = fsPacker.dumps(data, version=self.FSPACKER_VERSION).hex().encode('utf8')
+			elif self.requestProtocol == "RAW":
+				if type(data) is bytes:
+					payload = data
+				else:
+					payload = data.encode("utf8")
 			else:
 				raise RuntimeError
 			self._sendToSocket(payload)
 		else:
 			raise RuntimeError
-	def _sendToSocket(self, payload:bytes, path:str="/", extraHttpHeaders:Dict[str, str]={}) -> None:
+	def _sendToSocket(self, payload:bytes, httpMethod:str="POST", path:str="/", httpHeaders:Dict[str, str]={}) -> None:
 		if isinstance(self.socket, T_BaseClientSocket_send_http):
-			self.socket.send(payload, path, extraHttpHeaders)
+			self.socket.send(payload, httpMethod, path, httpHeaders)
 		elif isinstance(self.socket, T_BaseClientSocket_send_default):
 			self.socket.send(payload)
 		else:
@@ -210,6 +238,7 @@ class Client(T_Client):
 					ssl                = self.ssl,
 					sslHostname        = self.sslHostname,
 					extraHeaders       = self.extraHttpHeaders,
+					path               = self.path,
 					disableCompression = self.disableCompression,
 				)
 			elif self.messageProtocol == "STR":
@@ -276,42 +305,68 @@ class Client(T_Client):
 					self.socketErrors += 1
 					continue
 				raise
-	def _parseResponse(self, payload:bytes, headers:Optional[T_Headers]=None, charset:str="utf8") -> None:
+	def _parseResponse(self, payload:bytes, headers:Optional[T_Headers]=None, charset:str="utf8",
+	httpStatus:Optional[HTTPStatus]=None) -> None:
 		if self.requestProtocol in ["JSONRPC-2", "JSONRPC-P"]:
 			try:
 				data = json.loads(payload.decode(charset))
 			except:
-				raise MessageError("Invalid payload")
+				raise ResponseError("Invalid payload")
 			if not isinstance(data, list):
 				data = [data]
 			for r in data:
 				if not isinstance(r, dict):
-					raise MessageError("Invalid payload")
+					raise ResponseError("Invalid payload")
 				if "id" not in r:
-					raise MessageError("Required data missing: id")
+					raise ResponseError("Required data missing: id")
 				id = r["id"]
 				uid = r.get("uid", None)
 				isSuccess = not ("error" in r and r["error"])
 				if isSuccess:
 					if "result" not in r:
-						raise MessageError("Required data missing: result")
+						raise ResponseError("Required data missing: result")
 					result = r["result"]
 				else:
 					result = r["error"]
-				self._parseResult(id, isSuccess, result, uid)
+				self._parseResult(id, isSuccess, result, uid, httpStatus)
+		elif self.requestProtocol == "REST":
+			try:
+				data = json.loads(payload.decode(charset))
+			except:
+				raise ResponseError("Invalid payload")
+			self._parseResult(
+				min(( cast(int, x) for x in self.requests.keys() if not self.requests[x].isDone())),
+				True,
+				data,
+				"",
+				httpStatus,
+			)
+		elif self.requestProtocol == "RAW":
+			if self.messageProtocol == "STR":
+				try:
+					data = payload.decode('utf8')
+				except:
+					raise ResponseError("Invalid payload")
+			else:
+				data = payload
+			self._parseResult(
+				min(( cast(int, x) for x in self.requests.keys() if not self.requests[x].isDone())),
+				True,
+				data,
+				"",
+				httpStatus,
+			)
 		elif self.requestProtocol == "FSP":
 			if self.messageProtocol == "STR":
 				payload = bytes.fromhex(payload.decode(charset))
 			try:
 				_, data = fsPacker.loads(
 					payload,
-					maxDictSize=0,
-					maxOPSize=0,
 					maxIndexSize=0,
 					recursiveLimit=512,
 				)
 			except:
-				raise MessageError("Invalid payload")
+				raise ResponseError("Invalid payload")
 			if not isinstance(data, list):
 				data = [data]
 			for chunk in data:
@@ -321,7 +376,7 @@ class Client(T_Client):
 				(isinstance(chunk[3], str) or chunk[3] is None):
 					self._parseResult(*chunk)
 				else:
-					raise MessageError("Invalid payload")
+					raise ResponseError("Invalid payload")
 		elif self.requestProtocol == "OldFSProtocol":
 			if self.messageProtocol == "STR":
 				payload = bytes.fromhex(payload.decode(charset))
@@ -332,14 +387,14 @@ class Client(T_Client):
 			isinstance(resp[2], bytes):
 				self._parseResult(resp[0], resp[1], pickle.loads(decompress(resp[2])), "")
 			else:
-				raise MessageError("Invalid payload")
-	def _parseResult(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str) -> None:
+				raise ResponseError("Invalid payload")
+	def _parseResult(self, id:Union[int, str], isSuccess:bool, result:Any, uid:str, httpStatus:Optional[HTTPStatus]=None) -> None:
 		if id not in self.requests:
 			self.log.warn("Got unexpected result id: {}", id)
 			return
 		obj = self.requests[id]
 		self.log.debug("Received result for ID: {} UID: {}".format(id, uid))
-		obj._parseResponse( id, isSuccess, result, uid )
+		obj._parseResponse(id, isSuccess, result, uid, httpStatus)
 	def clear(self) -> None:
 		self.requests.clear()
 	def clone(self, **kwargs:Any) -> T_Client:
@@ -347,32 +402,45 @@ class Client(T_Client):
 		opts.update(kwargs)
 		return Client(**opts)
 	def close(self) -> None:
+		self.log.debug("Closing..")
 		if hasattr(self, "socket"):
 			self.socket.close()
 			del self.socket
 		self.clear()
 		self.log.debug("Closed")
 	__del__ = close
-	def request(self, method:str, args:List[Any]=[], kwargs:Dict[str, Any]={}, id:Optional[Union[str, int]]=None,
-	path:str="/") -> T_Request:
-		if id is None:
+	def request(self, method:str="", args:List[Any]=[], kwargs:Dict[str, Any]={}, id:Optional[Union[str, int]]=None,
+	path:Optional[str]=None, httpMethod:str="POST", httpHeaders:Optional[Dict[str, str]]=None,
+	payload:Any=NoPayload) -> T_Request:
+		if self.requestProtocol in ("REST", "RAW"):
 			id = self.id
 			self.id += 1
-		if type(id) not in [int, str]:
-			raise RequestError("Request ID can be only str or int")
-		if id in self.requests:
-			raise RequestError("Request ID already in use: {}".format(id))
+		else:
+			if id is None:
+				id = self.id
+				self.id += 1
+			if type(id) not in [int, str]:
+				raise RequestError("Request ID can be only str or int")
+			if id in self.requests:
+				raise RequestError("Request ID already in use: {}".format(id))
+		rHttpHeaders = {}
+		if httpHeaders:
+			for k, v in httpHeaders.items():
+				rHttpHeaders[k.lower()] = v
 		obj = Request(
 			self,
 			id,
 			method,
 			args,
 			kwargs,
-			path,
+			path or self.path,
+			httpMethod,
+			rHttpHeaders,
+			payload,
 			self.convertNumbers,
 		)
 		self.requests[id] = obj
 		self.log.info("Request queued: {} [{}]".format(method, id))
 		if hasattr(self, "socket"):
-			self._sendRequest( obj._dumps() )
+			self._sendRequest(obj._dumps(), httpMethod, path or self.path, rHttpHeaders)
 		return obj
