@@ -1,9 +1,11 @@
 # Builtin modules
-import re, ssl, traceback, socket, errno, codecs
+import re, ssl, traceback, socket, errno, codecs, weakref
+from io import BytesIO
 from time import monotonic
 from typing import Tuple, Dict, Union, Callable, NoReturn, Optional, cast
 from selectors import DefaultSelector, EVENT_READ, EVENT_WRITE
 from math import ceil
+from http import HTTPStatus
 # Third party modules
 import fsPacker
 # Local modules
@@ -22,7 +24,7 @@ class BaseClientSocket(T_BaseClientSocket):
 	bind:Optional[T_SocketBindAddress], connectTimeout:Union[int, float], transferTimeout:Union[int, float], ssl:bool,
 	sslHostname:Optional[str]) -> None:
 		assert protocol in ["TCPv4", "TCPv6", "IPC"], "Unsupported protocol"
-		self.client            = client
+		self.client            = weakref.proxy(client)
 		self.protocol          = protocol
 		self.target            = target
 		self.connectTimeout    = float(connectTimeout)
@@ -46,7 +48,7 @@ class BaseClientSocket(T_BaseClientSocket):
 			elif self.protocol == "IPC":
 				self.log.info("Connecting to {} ..".format(self.target))
 		cerr = self.sock.connect_ex(self.target)
-		if cerr in [errno.EAGAIN, errno.EINPROGRESS]: # errno.ENETUNREACH, errno.EADDRNOTAVAIL
+		if cerr in [errno.EAGAIN, errno.EINPROGRESS]:
 			return False
 		elif cerr in [0, errno.EISCONN]:
 			if self.ssl:
@@ -119,8 +121,6 @@ class BaseClientSocket(T_BaseClientSocket):
 			except OSError as err:
 				if err.errno == errno.EADDRINUSE:
 					self.log.warn("Bind address {} already in use, switch to automatic", self.bind)
-					# elif err.errno == errno.EADDRNOTAVAIL:
-					# 	self.log.warn("Bind address {} not available, switch to automatic", self.bind)
 				else:
 					self._raiseSocketError("Error during binding socket to {}: {}".format(self.bind, err))
 				self.bind = None
@@ -235,18 +235,22 @@ class BaseClientSocket(T_BaseClientSocket):
 		self.log.error(err)
 		raise MessageError(err) from None
 	def _reset(self) -> None:
+		self.log.debug("Reseting..")
 		self.readBuffer = b""
 		self.writeBuffer = b""
 		self.connectionStatus = NOT_CONNECTED
 		self.timeoutTimer = 0.0
 		if hasattr(self, "sock"):
+			self.log.trace("SOCKET SHUTDOWN")
 			try: self.sock.shutdown(socket.SHUT_RDWR)
 			except: pass
+			self.log.trace("SOCKET CLOSE")
 			try: self.sock.close()
 			except: pass
 			del self.sock
 		self.mask = EVENT_READ
 		self.sslTimer = 0.0
+		self.log.debug("Reseted")
 	def _setMask(self, newMask:int) -> None:
 		if self.sockFD and self.mask != newMask:
 			self.mask = newMask
@@ -259,7 +263,7 @@ class BaseClientSocket(T_BaseClientSocket):
 			self._setMask(EVENT_READ | EVENT_WRITE)
 	def close(self) -> None:
 		if hasattr(self, "sock"):
-			self.log.trace("Closing [FD:{}]", self.sockFD)
+			self.log.debug("Closing [FD:{}]", self.sockFD)
 			self.mask = EVENT_READ
 			self._reset()
 			self.log.debug("Closed")
@@ -275,11 +279,13 @@ class BaseClientSocket(T_BaseClientSocket):
 	def _isConnected(self) -> bool:
 		return self.connectionStatus == CONNECTED
 	def isAlive(self) -> bool:
-		return not (
-			(self.connectionStatus == NOT_CONNECTED) or
-			(self.connectionStatus == CONNECTING and monotonic()-self.timeoutTimer > self.connectTimeout) or
-			(self.connectionStatus == CONNECTED and monotonic()-self.timeoutTimer > self.transferTimeout)
-		)
+		if (
+				(self.connectionStatus == NOT_CONNECTED) or
+				(self.connectionStatus == CONNECTING and monotonic()-self.timeoutTimer > self.connectTimeout) or
+				(self.connectionStatus == CONNECTED and monotonic()-self.timeoutTimer > self.transferTimeout)
+			):
+			return False
+		return True
 	def loop(self, whileFn:Callable[[], bool]) -> None:
 		checkTimer = monotonic()
 		try:
@@ -310,13 +316,13 @@ class HTTPClientSocket(BaseClientSocket, T_HTTPClientSocket):
 	}
 	def __init__(self, client:T_Client, protocol:str, target:Union[str, Tuple[str, int], Tuple[str, int, int, int]],
 	bind:Optional[T_SocketBindAddress], connectTimeout:Union[int, float], transferTimeout:Union[int, float], ssl:bool=False,
-	sslHostname:Optional[str]=None, extraHeaders:Dict[str, str]={}, disableCompression:bool=False) -> None:
+	sslHostname:Optional[str]=None, extraHeaders:Dict[str, str]={}, path:str="/", disableCompression:bool=False) -> None:
 		super().__init__(client, protocol, target, bind, connectTimeout, transferTimeout, ssl, sslHostname)
+		self.path = path
 		self.headers = Headers(self.defaultHeaders)
 		self.headers.update(extraHeaders)
 		if not disableCompression:
-			self.headers.update({"Accept-Encoding":"deflate"})
-		return None
+			self.headers.update({"accept-encoding":"deflate"})
 	def parseReadBuffer(self) -> bool:
 		endLine = b"\r\n"
 		pos = self.readBuffer.find(endLine*2)
@@ -326,18 +332,20 @@ class HTTPClientSocket(BaseClientSocket, T_HTTPClientSocket):
 		elif pos > 4096:
 			self._raiseMessageError("HTTP headers are too long")
 		endLineLen = len(endLine)
-		while pos != -1:
+		while not self.signal.get() and pos != -1:
 			rawData = self.readBuffer[pos+endLineLen*2:]
 			rawHeaders = self.readBuffer[:pos].decode("ISO-8859-1").split(endLine.decode("ISO-8859-1"))
 			if not rawHeaders:
 				self._raiseMessageError("Invalid HTTP headers")
-			httpResponse = rawHeaders.pop(0).split(" ")
+			httpResponse = rawHeaders.pop(0).split(" ", 2)
 			if len(httpResponse) < 2:
 				self._raiseMessageError("Invalid HTTP response code")
-			# if httpResponse[1] == "503":
-			# 	self._raiseSocketError("Server offline")
-			# elif httpResponse[1] != "200":
-			# 	self._raiseSocketError("Request failure")
+			if not httpResponse[1].isdigit():
+				self._raiseMessageError("Invalid HTTP response code")
+			try:
+				responseHTTPStatus = HTTPStatus(int(httpResponse[1]))
+			except:
+				self._raiseMessageError("Invalid HTTP response code")
 			headers = Headers()
 			for rawHeader in rawHeaders:
 				s = rawHeader.find(":")
@@ -354,7 +362,9 @@ class HTTPClientSocket(BaseClientSocket, T_HTTPClientSocket):
 				if cLength < 0:
 					self._raiseMessageError("Invalid HTTP header value for content-length")
 			elif "chunked" not in headers.get("transfer-encoding", "").lower():
-				self._raiseMessageError("Invalid HTTP transfer encoding. Not chunked and no content-length given.")
+				self._raiseMessageError(
+					"Invalid HTTP transfer encoding. Content-Length is required when encoding is not chunked."
+				)
 			cEncoding = headers.get("content-encoding", "")
 			if cEncoding == "":
 				compression = False
@@ -375,33 +385,44 @@ class HTTPClientSocket(BaseClientSocket, T_HTTPClientSocket):
 			#
 			payload = b""
 			if cLength is None:
-				# Chunked: data_size:int + endline:bytes + data:bytes
-				rawDataCache = rawData
-				EOC = False
-				pos = rawDataCache.find(endLine)
-				cLength = 0
-				while pos != -1:
-					cLength += endLineLen
-					if pos == 0:
-						# end of chunks
-						EOC = True
-						break
-					rawChunk = rawDataCache[:pos]
-					rawDataCache = rawDataCache[pos+endLineLen:]
+				rawDataCache = BytesIO(rawData)
+				chunkLength = 0
+				while not self.signal.get():
+					# we read the chunk length
+					rawChunkLength = rawDataCache.readline()
+					if rawChunkLength[-2:] != b"\r\n":
+						# We waiting for the next chunk
+						return False
+						# self._raiseMessageError("Invalid HTTP chunk size")
+					if len(rawChunkLength) > 10:
+						self._raiseMessageError("HTTP chunk too big")
+					# Converting from hexa to int
 					try:
-						chunkLength = int(rawChunk, 16)
+						chunkLength = int(rawChunkLength[:-2], 16)
 					except:
 						self._raiseMessageError("Invalid HTTP chunk length")
-					if chunkLength > 0xFFFFFF:
+					# Maximum limit of a chunk
+					if chunkLength > 0xFFFFFFFF:
 						self._raiseMessageError("HTTP chunk too big")
-					if chunkLength > len(rawDataCache):
+					# When the length is zero, then the data is over
+					if chunkLength == 0:
+						cLength = rawDataCache.tell()
+						# Maybe the server puts an `\r\n `end to the data
+						if rawDataCache.read(2) == b"\r\n":
+							cLength = rawDataCache.tell()
+						break
+					# Read the whole chunk from the buffer
+					chunk = rawDataCache.read(chunkLength)
+					# Compare the length what we readed, when not equals then we did not have the whole data
+					if len(chunk) != chunkLength:
 						return False
-					payload += rawDataCache[:chunkLength]
-					cLength += chunkLength
-					rawDataCache = rawDataCache[chunkLength:]
-					pos = rawDataCache.find(endLine)
-				if not EOC:
-					return False
+					payload += chunk
+					# we read the chunk end indicator `\r\n` from the end
+					eoc = rawDataCache.read(2)
+					if len(eoc) != 2:
+						return False
+					if eoc != b"\r\n":
+						self._raiseMessageError("Invalid HTTP chunk data")
 			else:
 				if len(rawData) < cLength:
 					return False
@@ -416,11 +437,11 @@ class HTTPClientSocket(BaseClientSocket, T_HTTPClientSocket):
 					self._raiseMessageError("Invalid response content")
 			if self.log.isFiltered("TRACE"):
 				self.log.trace("Payload [len: {}]: {}", len(payload), payload)
-			self.client._parseResponse(payload, headers, charset)
+			self.client._parseResponse(payload, headers, charset, responseHTTPStatus)
 			pos = self.readBuffer.find(endLine*2)
 		return False
-	def send(self, payload:bytes=b"", path:str="/", headers:Dict[str, str]={}) -> None:
-		rawHeader = "{} {} HTTP/1.1\r\n".format("POST" if payload else "GET", path)
+	def send(self, payload:bytes=b"", httpMethod:str="POST", path:Optional[str]=None, headers:Dict[str, str]={}) -> None:
+		rawHeader = "{} {} HTTP/1.1\r\n".format(httpMethod, path or self.path)
 		if payload:
 			headers["content-length"] = str(len(payload))
 		rawHeader += self.headers.dumps(extend=headers)
