@@ -1,6 +1,7 @@
 # Builtin modules
 import os, unittest
 from time import monotonic
+from http import HTTPStatus
 # Third party modules
 from fsLogger import SimpleLogger, Logger
 from fsSignal import Signal, SoftSignal
@@ -46,6 +47,7 @@ class ClientTest(unittest.TestCase):
 				"data": "ping() takes 0 positional argument but 1 were given",
 				"message": "Invalid params"
 			})
+			assert(r.getHTTPStatus() == HTTPStatus.OK)
 			r = c.request("surenoteexists")
 			self.assertEqual(r.isSuccess(), False)
 			self.assertEqual(r.isDone(), True)
@@ -161,7 +163,7 @@ class ClientTest(unittest.TestCase):
 			retryCount=2,
 			retryDelay=1,
 			log=Logger("Client"),
-			signal=SoftSignal()
+			signal=SoftSignal(),
 		)
 		s = monotonic()
 		r = rootClient.request("ping")
@@ -170,3 +172,12 @@ class ClientTest(unittest.TestCase):
 		except:
 			pass
 		self.assertGreater(monotonic()-s, 1.0)
+	def test_binance(self) -> None:
+		with Client(
+			"TCPv4:HTTP:REST",
+			("fapi.binance.com", 443),
+			ssl=True,
+			log=Logger("Client"),
+			signal=SoftSignal(),
+		) as clnt:
+			clnt.request(httpMethod="GET", path="/fapi/v1/premiumIndex").get()
